% Empirical semivariogram 2D
% Lags that have less than 30 pairs are ignored
%
% INPUT:
% Number of Classes 
% Direction 
% Tolerance (=pi for isotropic)
% Coordinates X and Y
% Fluctuation Diak
% flag (Do we want the figure?) 1=yes
% MDIS Maximum Variogram cutoff distance COMPARED to maximum pair distance. 
%   I.e. MDIS=0.5 means the maximum distance is set as half the maximum
%   distance of the pairs. (Default = 0.425)
%
%
% [Variogram, lagcent, Pairsperclass]=empirvar(Lags,f,tole, X, Y, Diak, flag, MDIS);
%
% OUTPUT:
% Variogram
% Location of the center of the lag classes
% Number of pairs in each class
% NOTE: The program does NOT work well for very small distances.
% NOTE: The program works for Directions between (0, pi/2)
%
%
% Created by Andy Pavlides, November 2016 for the Geostatistics Course of
% University of Liege

function [Variogram, distance, Pairsperclass]=empirvar(Lags,f,tole, X2, Y2, Diak, flag, MDIS);
lags=Lags;

if nargin<8
    MDIS=0.425
end
if MDIS>1
    MDIS=1;
end

f2=f*360/2/pi;
f3=(f-pi/2)*360/2/pi;
tole2=tole*360/2/pi;
Dir1=num2str(f2);
Dir2=num2str(f3);
Direct=strcat(' Direction:',Dir1);
Direct2=strcat(' Direction:',Dir2);
Tole1=num2str(tole2);
Toleran=strcat(' Tolerance:',Tole1);
Titl=strcat(Direct,',',Toleran);
Tit2=strcat(Direct2,',',Toleran);

t0=cputime;

n=length(Diak);
maxtim=max(Diak);

X=[zeros(length(Diak),1) X2 Y2 Diak];

[n m]=size(X);
nx=X(:,2);
ny=X(:,3);
value=X(:,4);

% ************ ANGLE TOLERANCE ******************
minf=f-tole;
maxf=f+tole;

% ************ DISTANCE **************
% AP the distance of each point with each other

AP=pdist2([X2 Y2],[X2 Y2]);

% ************ LAG CALCULATION **************** 

rcl=zeros((Lags+1),1);
MAP=MDIS*max(max(AP));
lagd=MAP/Lags/2;
%lagd the radius of each class 
%MAP maximum distance
        
% Kentra
rcl(1)=0;
for i=2:Lags+1
    rcl(i)=rcl(i-1)+2*lagd;
end

% ********* LAG CALCULATION *******
class=zeros(n,n);
t=zeros(n,n);
prwtovar=zeros(2*(Lags+1),1);
pointsinclass=zeros(Lags+1,1);
BG(n,n)=NaN;
for i=1:n
    for j=1:n
        class(i,j)=round(AP(i,j)/(2*lagd))+1;
        if class (i,j)<Lags+2
            dx=nx(j)-nx(i);
            dy=ny(j)-ny(i)+0.000001;
            t(i,j)=acot(dx/dy+0.000001);
            if minf<t(i,j)
                if t(i,j)<maxf
                    if AP(i,j)>eps
                        pointsinclass(class(i,j))=pointsinclass(class(i,j))+1;
                        prwtovar(class(i,j))=prwtovar(class(i,j))+(abs(value(i)-value(j)))^0.5;
                    end
                end
            end
            if minf-pi<t(i,j)
                if t(i,j)<maxf-pi
                    if AP(i,j)>eps
                        pointsinclass(class(i,j))=pointsinclass(class(i,j))+1;
                        prwtovar(class(i,j))=prwtovar(class(i,j))+(abs(value(i)-value(j)))^0.5;
                    end
                end
            end
        end
    end
end

% ********* VARIOGRAM CALCULATION **********
% I'm using the ROBUST variogram estimator! ! !

var=zeros(Lags+1,1);
secvar=zeros(Lags+1,1);

for i=1:(Lags+1)
    secvar(i)=(prwtovar(i)/pointsinclass(i))^4;
    var(i)=secvar(i)/(0.494/pointsinclass(i)+0.457)/2;
    if pointsinclass(i)<30
        var(i)=NaN;
    end
end
distance=zeros(Lags+1,1);
for i=2:Lags+1
    distance(i)=(i-1)*2*lagd;
end
DDD(1:Lags+1)=0;
for i=1:2:2*Lags+2
    distance2(i/2+0.5)=i*lagd;
end


Variogram=var;
Variogram(1)=nan;
Pairsperclass=pointsinclass;

if nargin>6 && flag==1
    
fig=figure;
axes1 = axes('Parent',fig, 'fontsize',16);
hold(axes1,'all');
plot(distance,Variogram,'ro','MarkerFaceColor','r','MarkerSize',5);
hold on;
plot(0,0,'k.');
xlabel('distance')
ylabel('\gamma(r)')
hold off;

end 

% ********* KA8ARISMOS VARIOGRAMMATOS **********
% Removing nan
G=isnan(Variogram);
Variogram(G==1)=[];
distance(G==1)=[];
Pairsperclass(G==1)=[];

Tol=cputime-t0;