%	WLS_SSRF computes the WLS distance between the empirical variogram and
%	the Spartan semivariogram model. 
%    WLS_SSRF weights the variogram distance by the ratio of the number of
%   pairs divided by the square of 
%	the model variogram (DIfferent weighting schemes can be used.)
%
%   INPUT
%
%   Param:  SSRF Variogram parameters [eta0 eta1 xi nugget]
%   lags:   Array of lag vectors
%   varioz: Array of semivariogram values at respective lags
%   npair:  Number of variogram pairs per lag
%   Weighting: 1 Number of pairs. 2 inverse of square empirical variogram
%           if no weighting option is used, just least squares are used
%
%	OUTPUT:
%
%	Y:		Value of objective function
%   [y] = wls_ssrf(Param,lags,varioz,npair, Weighting)
%
%==========================================================================
%
%  Copyright (C) 2016  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================

function [y] = wls_ssrf(Param,lags,varioz,npair,Weighting)

if nargin<5
    Weighting=0;
end

nparam=length(Param);

[~,S0]=cov_ssrf_exp(3,eps,Param,0);
[~,Cr]=cov_ssrf_exp(3,lags,Param,0);

if nparam==4
    variom = S0 - Cr +Param(4); 
else
    variom = S0 - Cr; 
end

if Weighting==1
    y=sum(npair.*(varioz-variom).^2);
elseif Weighting==2
    y=sum(1./varioz .*(varioz-variom).^2);
else
    y=sum((varioz-variom).^2);
end

